public inbox for linux-kernel@vger.kernel.org
 help / color / mirror / Atom feed
* [patch 1/3] x86, cpa: fix kernel text RO checks in static_protection()
@ 2009-10-29  2:46 Suresh Siddha
  2009-10-29  2:05 ` Steven Rostedt
                   ` (3 more replies)
  0 siblings, 4 replies; 10+ messages in thread
From: Suresh Siddha @ 2009-10-29  2:46 UTC (permalink / raw)
  To: mingo, hpa, linux-kernel; +Cc: Suresh Siddha, rostedt

[-- Attachment #1: fix_cpa_check.patch --]
[-- Type: text/plain, Size: 3167 bytes --]

Steven Rostedt reported that we are unconditionally making the
kernel text mapping as read-only. i.e., if someone does cpa() to the
kernel text area for setting/clearing any page table attribute, we
unconditionally clear the read-write attribute for the kernel text mapping
that is set at compile time.

We should delay (to forbid the write attribute) and enforce only after the
kernel has mapped the text as read-only.

Reported-by: Steven Rostedt <rostedt@goodmis.org>
Signed-off-by: Suresh Siddha <suresh.b.siddha@intel.com>
---
 arch/x86/include/asm/cacheflush.h |    1 +
 arch/x86/mm/init_32.c             |    2 +-
 arch/x86/mm/init_64.c             |    2 +-
 arch/x86/mm/pageattr.c            |   10 ++++++----
 4 files changed, 9 insertions(+), 6 deletions(-)

Index: tip/arch/x86/include/asm/cacheflush.h
===================================================================
--- tip.orig/arch/x86/include/asm/cacheflush.h
+++ tip/arch/x86/include/asm/cacheflush.h
@@ -176,6 +176,7 @@ void clflush_cache_range(void *addr, uns
 #ifdef CONFIG_DEBUG_RODATA
 void mark_rodata_ro(void);
 extern const int rodata_test_data;
+extern int kernel_set_to_readonly;
 void set_kernel_text_rw(void);
 void set_kernel_text_ro(void);
 #else
Index: tip/arch/x86/mm/init_32.c
===================================================================
--- tip.orig/arch/x86/mm/init_32.c
+++ tip/arch/x86/mm/init_32.c
@@ -997,7 +997,7 @@ static noinline int do_test_wp_bit(void)
 const int rodata_test_data = 0xC3;
 EXPORT_SYMBOL_GPL(rodata_test_data);
 
-static int kernel_set_to_readonly;
+int kernel_set_to_readonly;
 
 void set_kernel_text_rw(void)
 {
Index: tip/arch/x86/mm/init_64.c
===================================================================
--- tip.orig/arch/x86/mm/init_64.c
+++ tip/arch/x86/mm/init_64.c
@@ -695,7 +695,7 @@ void __init mem_init(void)
 const int rodata_test_data = 0xC3;
 EXPORT_SYMBOL_GPL(rodata_test_data);
 
-static int kernel_set_to_readonly;
+int kernel_set_to_readonly;
 
 void set_kernel_text_rw(void)
 {
Index: tip/arch/x86/mm/pageattr.c
===================================================================
--- tip.orig/arch/x86/mm/pageattr.c
+++ tip/arch/x86/mm/pageattr.c
@@ -282,14 +282,16 @@ static inline pgprot_t static_protection
 #if defined(CONFIG_X86_64) && defined(CONFIG_DEBUG_RODATA) && \
 	!defined(CONFIG_DYNAMIC_FTRACE)
 	/*
 	 * Kernel text mappings for the large page aligned .rodata section
 	 * will be read-only. For the kernel identity mappings covering
 	 * the holes caused by this alignment can be anything.
 	 *
 	 * This will preserve the large page mappings for kernel text/data
 	 * at no extra cost.
 	 */
-	if (within(address, (unsigned long)_text,
+	if (kernel_set_to_readonly &&
+	    within(address, (unsigned long)_text,
 		   (unsigned long)__end_rodata_hpage_align))
 		pgprot_val(forbidden) |= _PAGE_RW;
 #endif



^ permalink raw reply	[flat|nested] 10+ messages in thread

end of thread, other threads:[~2009-11-02 18:56 UTC | newest]

Thread overview: 10+ messages (download: mbox.gz follow: Atom feed
-- links below jump to the message on this page --
@@ -282,14 +282,16 @@ static inline pgprot_t static_protection
 	!defined(CONFIG_DYNAMIC_FTRACE)
 	/*
-	 * Kernel text mappings for the large page aligned .rodata section
-	 * will be read-only. For the kernel identity mappings covering
-	 * the holes caused by this alignment can be anything.
+	 * Once the kernel maps the text as RO (kernel_set_to_readonly is set),
+	 * kernel text mappings for the large page aligned text, rodata sections
+	 * will be always read-only. For the kernel identity mappings covering
+	 * the holes caused by this alignment can be anything that user asks.
 	 *
 	 * This will preserve the large page mappings for kernel text/data
2009-10-29  2:46 [patch 1/3] x86, cpa: fix kernel text RO checks in static_protection() Suresh Siddha
2009-10-29  2:05 ` Steven Rostedt
2009-10-29  2:46 ` [patch 2/3] x86_64, ftrace: make ftrace use kernel identity mapping to modify code Suresh Siddha
2009-11-02 18:54   ` [tip:x86/mm] x86_64, ftrace: Make " tip-bot for Suresh Siddha
2009-10-29  2:46 ` [patch 3/3] x86_64, cpa: use only text section in set_kernel_text_rw/ro Suresh Siddha
2009-10-29  2:06   ` Steven Rostedt
2009-10-30 19:17     ` Suresh Siddha
2009-10-30 19:59       ` Steven Rostedt
2009-11-02 18:55   ` [tip:x86/mm] x86_64, cpa: Use " tip-bot for Suresh Siddha
2009-11-02 18:54 ` [tip:x86/mm] x86, cpa: Fix kernel text RO checks in static_protection() tip-bot for Suresh Siddha

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox